/*
FALLOFF
compute probability falloff
*/

// solidity falloff mode
final int FALLOFF_CONSTANT = 0;
final int FALLOFF_LINEAR = 1;
final int FALLOFF_SQUARED = 2;
final int FALLOFF_COSINE = 3;
final int FALLOFF_COSINE_SQ = 4;
final int FALLOFF_RADIAL = 5;
final int FALLOFF_RADIAL_SQ = 6;
final int FALLOFF_RADIAL_COSINE = 7;
final int FALLOFF_RADIAL_COSINE_SQ = 8;
public final String[] prob_name =
{
  "constant",
  "linear",
  "squared",
  "cosine",
  "cosine^2",
  "radial",
  "radial^2",
  "radial cosine",
  "radial cosine^2"
};

// selected falloff mode 
public int falloff_mode = FALLOFF_RADIAL_SQ;

// compute the probability table for a given falloff mode
// (each element is the probability of the pixel being solid)
public float[][] compute_falloff(int mode, int size)
{
  float[][] prob = new float[size][size];
  
  final float x_scale = 2.0 / size;
  final float y_scale = 2.0 / size;

  for (int iy = 0; iy < size; ++iy)
  {
    final float fy = (iy + 0.5f) * y_scale - 1;
    
    for (int ix = 0; ix < size; ++ix)
    {
      final float fx = (ix + 0.5f) * x_scale - 1;

      // get probability value for this pixel being solid
      // taking into account falloff function and value range      
      float value;
      switch (mode)
      {
      default:
      case FALLOFF_CONSTANT:
        // constant across the tile
        value = 1;
        break;
      case FALLOFF_LINEAR:
        //  linear falloff along each axis
        value = (1 - abs(fx)) * (1 - abs(fy));
        break;
      case FALLOFF_SQUARED:
        //  quadratic falloff along each axis
        value = (1 - fx * fx) * (1 - fy * fy);
        break;
      case FALLOFF_COSINE:
        // cosine falloff along each axis
        value = cos(HALF_PI * fx) * cos(HALF_PI * fy);
        break;
      case FALLOFF_COSINE_SQ:
        // cosine squared falloff along each axis
        value = (0.5 + 0.5 * cos(PI * fx)) * (0.5 + 0.5 * cos(PI * fy));
        break;
      case FALLOFF_RADIAL:
        // linear falloff from the center
        value = 1 - constrain(sqrt(fx * fx + fy * fy), 0, 1);
        break;
      case FALLOFF_RADIAL_SQ:
        // quadratic falloff from the center
        value = 1 - constrain(fx * fx + fy * fy, 0, 1);
        break;
      case FALLOFF_RADIAL_COSINE:
        // cosine falloff from the center
        value = cos(HALF_PI * constrain(sqrt(fx * fx + fy * fy), 0, 1));
        break;
      case FALLOFF_RADIAL_COSINE_SQ:
        // cosine squared falloff from the center
        value = 0.5 + 0.5 * cos(PI * constrain(sqrt(fx * fx + fy * fy), 0, 1));
        break;
      }
      prob[iy][ix] = min_prob + (max_prob - min_prob) * value;
    }
  }
  
  return prob;
}

