/*
NOISE
This is a simplified version of the noise function used by Processing.
*/

// fill a noise table with uniform random values between 0 and 1
public void generate_noise(float[][] noise, int seed)
{
  randomSeed(seed);
  random(1);  
  for (int iy = 0; iy < noise.length; ++iy)
  {
    for (int ix = 0; ix < noise[iy].length; ++ix)
    {
      noise[iy][ix] = random(1);
    }
  }
}

// smooth step function
float smooth_step(float t)
{
  return t * t * (3 - 2 * t);
}

// evaluate noise at position (x, y)
// the noise table must be at least floor(x)+1 by floor(y)+1
public float eval_noise(float[][] noise, float x, float y)
{
  final int ix = int(floor(x));
  final int iy = int(floor(y));
  final float fx = smooth_step(x - ix);
  final float fy = smooth_step(y - iy);
  final float noise00 = noise[iy][ix]; 
  final float noise01 = noise[iy][ix+1]; 
  final float noise0 = noise00 + fx * (noise01 - noise00);         
  final float noise10 = noise[iy+1][ix]; 
  final float noise11 = noise[iy+1][ix+1];
  final float noise1 = noise10 + fx * (noise11 - noise10);         
  return noise0 + fy * (noise1 - noise0);
}
